package javax.swing.plaf.basic;

import java.beans.PropertyChangeEvent;
import javax.swing.UIManager;
import java.awt.Insets;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Font;
import javax.swing.text.View;
import java.awt.Rectangle;
import sun.swing.SwingUtilities2;
import java.awt.Graphics;
import javax.swing.JToolTip;
import javax.swing.LookAndFeel;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.ToolTipUI;

public class BasicToolTipUI extends ToolTipUI
{
    static BasicToolTipUI sharedInstance;
    private static PropertyChangeListener sharedPropertyChangedListener;
    private PropertyChangeListener propertyChangeListener;
    
    public static ComponentUI createUI(JComponent component) {
        return BasicToolTipUI.sharedInstance;
    }
    
    public BasicToolTipUI() {
    }
    
    @Override
    public void installUI(JComponent component) {
        this.installDefaults(component);
        this.installComponents(component);
        this.installListeners(component);
    }
    
    @Override
    public void uninstallUI(JComponent component) {
        this.uninstallDefaults(component);
        this.uninstallComponents(component);
        this.uninstallListeners(component);
    }
    
    protected void installDefaults(JComponent component) {
        LookAndFeel.installColorsAndFont(component, "ToolTip.background", "ToolTip.foreground", "ToolTip.font");
        LookAndFeel.installProperty(component, "opaque", Boolean.TRUE);
        this.componentChanged(component);
    }
    
    protected void uninstallDefaults(JComponent c) {
        LookAndFeel.uninstallBorder(c);
    }
    
    private void installComponents(JComponent component) {
        BasicHTML.updateRenderer(component, ((JToolTip)component).getTipText());
    }
    
    private void uninstallComponents(JComponent component) {
        BasicHTML.updateRenderer(component, "");
    }
    
    protected void installListeners(JComponent component) {
        component.addPropertyChangeListener(this.propertyChangeListener = this.createPropertyChangeListener(component));
    }
    
    protected void uninstallListeners(JComponent component) {
        component.removePropertyChangeListener(this.propertyChangeListener);
        this.propertyChangeListener = null;
    }
    
    private PropertyChangeListener createPropertyChangeListener(JComponent component) {
        if (BasicToolTipUI.sharedPropertyChangedListener == null) {
            BasicToolTipUI.sharedPropertyChangedListener = new BasicToolTipUI.PropertyChangeHandler();
        }
        return BasicToolTipUI.sharedPropertyChangedListener;
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        Font font = component.getFont();
        FontMetrics fontMetrics = SwingUtilities2.getFontMetrics(component, graphics, font);
        Dimension size = component.getSize();
        graphics.setColor(component.getForeground());
        String tipText = ((JToolTip)component).getTipText();
        if (tipText == null) {
            tipText = "";
        }
        Insets insets = component.getInsets();
        Rectangle o = new Rectangle(insets.left + 3, insets.top, size.width - (insets.left + insets.right) - 6, size.height - (insets.top + insets.bottom));
        View view = (View)component.getClientProperty("html");
        if (view != null) {
            view.paint(graphics, o);
        }
        else {
            graphics.setFont(font);
            SwingUtilities2.drawString(component, graphics, tipText, o.x, o.y + fontMetrics.getAscent());
        }
    }
    
    @Override
    public Dimension getPreferredSize(JComponent c) {
        FontMetrics fontMetrics = c.getFontMetrics(c.getFont());
        Insets insets = c.getInsets();
        Dimension dimension = new Dimension(insets.left + insets.right, insets.top + insets.bottom);
        String tipText = ((JToolTip)c).getTipText();
        if (tipText != null && !tipText.equals("")) {
            View view = (c != null) ? ((View)c.getClientProperty("html")) : null;
            if (view != null) {
                dimension.width += (int)view.getPreferredSpan(0) + 6;
                dimension.height += (int)view.getPreferredSpan(1);
            }
            else {
                dimension.width += SwingUtilities2.stringWidth(c, fontMetrics, tipText) + 6;
                dimension.height += fontMetrics.getHeight();
            }
        }
        return dimension;
    }
    
    @Override
    public Dimension getMinimumSize(JComponent component) {
        Dimension preferredSize = this.getPreferredSize(component);
        View view = (View)component.getClientProperty("html");
        if (view != null) {
            preferredSize.width = (int)((float)preferredSize.width - (view.getPreferredSpan(0) - view.getMinimumSpan(0)));
            return preferredSize;
        }
        return preferredSize;
    }
    
    @Override
    public Dimension getMaximumSize(JComponent component) {
        Dimension preferredSize = this.getPreferredSize(component);
        View view = (View)component.getClientProperty("html");
        if (view != null) {
            preferredSize.width = (int)((float)preferredSize.width + (view.getMaximumSpan(0) - view.getPreferredSpan(0)));
            return preferredSize;
        }
        return preferredSize;
    }
    
    private void componentChanged(JComponent component) {
        JComponent component2 = ((JToolTip)component).getComponent();
        if (component2 != null && !component2.isEnabled()) {
            if (UIManager.getBorder("ToolTip.borderInactive") != null) {
                LookAndFeel.installBorder(component, "ToolTip.borderInactive");
            }
            else {
                LookAndFeel.installBorder(component, "ToolTip.border");
            }
            if (UIManager.getColor("ToolTip.backgroundInactive") != null) {
                LookAndFeel.installColors(component, "ToolTip.backgroundInactive", "ToolTip.foregroundInactive");
            }
            else {
                LookAndFeel.installColors(component, "ToolTip.background", "ToolTip.foreground");
            }
        }
        else {
            LookAndFeel.installBorder(component, "ToolTip.border");
            LookAndFeel.installColors(component, "ToolTip.background", "ToolTip.foreground");
        }
    }
    
    static void access$100(BasicToolTipUI basicToolTipUI, JComponent component) {
        basicToolTipUI.componentChanged(component);
    }
    
    static {
        BasicToolTipUI.sharedInstance = new BasicToolTipUI();
    }
    
    private static class PropertyChangeHandler implements PropertyChangeListener
    {
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            String propertyName = propertyChangeEvent.getPropertyName();
            if (propertyName.equals("tiptext") || "font".equals(propertyName) || "foreground".equals(propertyName)) {
                JToolTip toolTip2 = (JToolTip)propertyChangeEvent.getSource();
                BasicHTML.updateRenderer(toolTip2, toolTip2.getTipText());
            }
            else if ("component".equals(propertyName)) {
                JToolTip toolTip = (JToolTip)propertyChangeEvent.getSource();
                if ((toolTip.getUI() instanceof BasicToolTipUI)) {
                    BasicToolTipUI.access$100((BasicToolTipUI)toolTip.getUI(), toolTip);
                }
            }
        }
    }
}
